/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 * 
 * Copyright (C) 2012  Eva Mujdricza-Maydt
 * Copyright (C) 2012  Huiqin Körkel-Qu
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Developers:
 * - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de)
 * - Huiqin Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */

package annotators.treetagger;

import java.util.ArrayList;
import objects.Sentence;
import objects.Word;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/** Handler for the xml-format output of the TreeTagger.
 *
 * @author Eva Mujdricza-Maydt
 * @lastChange 20120626
 */
public class XMLHandler_treeTaggerOutput extends DefaultHandler {

	private ArrayList<Sentence> sentences = null;
	private Sentence currSent = null;
	private boolean withinSent = false;

	//because of "character chunking"
	//see also: http://www.tutego.de/blog/javainsel/2007/01/character-chunking-bei-sax.html
	private static StringBuilder text = null;

	@Override
	public void startDocument() throws SAXException {
		sentences = new ArrayList<Sentence>();
	}


	@Override
	public void endDocument() throws SAXException {
		ExtractCorpus_forTreeTagger.sentencesInDoc = sentences;
	}

	@Override
	public void startElement(String namespaceURI, String localName,
		String qName, Attributes atts) throws SAXException {

		if (qName.equals(Constants_TreeTagger.ELEMENT_s)) {
			currSent = new Sentence();
			
			String val = atts.getValue(Constants_TreeTagger.ATTRIBUTE_id);
			if (val == null) {
				currSent = null;
			}
			else {
				currSent.setSentenceNr(Integer.valueOf(atts.getValue(Constants_TreeTagger.ATTRIBUTE_id)));

				text = new StringBuilder();
				withinSent = true;
			}
		}
	}


	@Override
	public void endElement(String namespaceURI, String localName,
		String qName) throws SAXException {

		if (qName.equals(Constants_TreeTagger.ELEMENT_s) && currSent != null) {

			ArrayList<Word> currWords = new ArrayList<Word>();
			
			if (text.toString().trim().isEmpty()) {
				//nothing, the currWords-list keeps empty
			} else {
				String[] textArray = text.toString().trim().split("\n");

				for (int i = 0; i < textArray.length; i++) {
					String line = textArray[i];
					String[] lineArray = line.trim().split("\t");
					if (lineArray.length != 3) {
						String msg = "\nLine (" + i + ") was not read correctly: \"" + line + "\" in text:\n" + text.toString() + "\n";
						throw new NullPointerException(msg);
					}
					String token = lineArray[0];
					String pos = lineArray[1];
					String lemma = lineArray[2];

					Word currWord = new Word();
					currWord.setWord(token);
					currWord.setPosTag(pos);
					currWord.setLemma(lemma);
					currWord.setSentenceNr(currSent.getSentenceNr());
					currWord.setWordInSentNr(i+1);

					currWords.add(currWord);
				}
			}
				
			currSent.setWordList(currWords);
			sentences.add(currSent);
		
			text = null;
			currWords = null;
			currSent = null;
			withinSent = false;
		}
	}


	@Override
	public void characters(char[] ch, int startPos, int length) {
		if (withinSent) {
			String t = new String(ch, startPos, length);
			text.append(t);
		}
	}

}
