/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 * 
 * Copyright (C) 2012  Eva Mujdricza-Maydt
 * Copyright (C) 2012  Huiqin Körkel-Qu
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Developers:
 * - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de)
 * - Huiqin Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
 
package corpora.opensubtitles;

import java.util.ArrayList;
import objects.Alignment_DocLevel;
import objects.Alignment_SentLevel;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/** Handler for one movie file pair from the provided alignment xml-file.
 *
 * @author Eva Mujdricza-Maydt
 * @lastChange 20120626
 */
public class XMLHandler_opensubtitles_alignmentsXML_oneDocPair extends DefaultHandler {

	private Alignment_DocLevel documentAlignment; //for one document
	private String givenSrcDocName;
	private boolean inGivenDoc = false;
	

	@Override
	public void startDocument() throws SAXException {
		givenSrcDocName = corpora.ExtractCorpus.srcFileNameForAligning;
	}


	@Override
	public void endDocument() throws SAXException {
		if (documentAlignment == null) {
			String msg = "\nAlignments for the given file pair could not be initialized.\n"
				+ "Probably there are no alignments for source file name \"" + givenSrcDocName + "\" in the input xml file.";
			throw new NullPointerException(msg);
		}
		corpora.ExtractCorpus.corpusAlignment_oneDoc = documentAlignment;
	}


	@Override
	public void startElement(String namespaceURI, String localName,
		String qName, Attributes atts) throws SAXException {

		if ((inGivenDoc == false) && qName.equals(Constants_OpenSubtitles.ELEMENT_linkGrp)) {

			String sourceDoc = atts.getValue(Constants_OpenSubtitles.ATTRIBUTE_fromDoc);
			if (sourceDoc.equals(givenSrcDocName)) {
				inGivenDoc = true;
				String targetDoc = atts.getValue(Constants_OpenSubtitles.ATTRIBUTE_toDoc);
				documentAlignment = new Alignment_DocLevel(sourceDoc, targetDoc);
			}
			
		}
		else if ((inGivenDoc == true) && qName.equals(Constants_OpenSubtitles.ELEMENT_link)) {

			//fill document-level alignment with links between sentences
			Alignment_SentLevel alSent = Alignment_SentLevel.newInstance();
			String raw_link = atts.getValue(Constants_OpenSubtitles.ATTRIBUTE_xtargets);
			int separatorIndex = raw_link.indexOf(Constants_OpenSubtitles.SEPARATOR_opus_link);

			ArrayList<Integer> sn = extractIntegerAL(
				raw_link.substring(0, separatorIndex));
			if (sn == null) {
				throw new NullPointerException("Null source alignment from link-string: " + raw_link);
			}
			alSent.setSourceSentAlign(sn);

			String ts = (separatorIndex == (raw_link.length()-1)) ? "" : raw_link.substring(separatorIndex+1);
			ArrayList<Integer> tn = extractIntegerAL(ts);
			if (tn == null) {
				throw new NullPointerException("Null target alignment from link-string: " + raw_link);
			}
			alSent.setTargetSentAlign(tn);

			documentAlignment.addSentAlignment(alSent);
		}
	}

	@Override
	public void endElement(String namespaceURI, String localName,
		String qName) throws SAXException {
		
		if ((inGivenDoc == true) && qName.equals(Constants_OpenSubtitles.ELEMENT_linkGrp)) {
			inGivenDoc = false;
		}
	}

	@Override
	public void characters(char[] ch, int startPos, int length) {
	}
	
	private static ArrayList<Integer> extractIntegerAL(String rawList) {
		ArrayList<Integer> al = new ArrayList<Integer>();

		if (rawList == null) {
			String msg = "\nNo integer raw string extracted.\n";
			throw new NullPointerException(msg);
		} else if(rawList.isEmpty()) {
			//nothing to do, the empty list will be returned
		} else {
			String[] rawListArray = rawList.split(" ");
			for (String s : rawListArray) {
				al.add(Integer.parseInt(s.trim()));
			}
		}

		return al;
	}
	
}
