/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;

/**
 * get the following information from file token and the corresponding Integer,
 * this is used to change a file into an array of int[].
 *
 * @author Huiqin Körkel-Qu
 */
public class getMultiWordsHash {

	public static HashMap<String, Integer> getWordsNumHash(String fileName) throws IOException {

		File wordsFile = new File(fileName);

		BufferedReader BR = new BufferedReader(new FileReader(wordsFile));
		HashMap<String, Integer> multiWordNumHash = new HashMap<String, Integer>();

		String currentLine;
		while (null != (currentLine = BR.readLine())) {

			String[] oneLine = currentLine.split("\\s+");
			multiWordNumHash.put(oneLine[1], Integer.parseInt(oneLine[0]));
//			System.out.println(oneLine[1]+" "+oneLine[0]);
		}

		BR.close();
		return multiWordNumHash;
	}

	/**
	 * get the following information from file Integer(which represents a
	 * token), and how many times the corresponding token appears
	 *
	 * @param fileName
	 * @return
	 * @throws IOException
	 */
	public static HashMap<Integer, Integer> getMultiNumTimesHash(String fileName) throws IOException {

		File wordsFile = new File(fileName);

		BufferedReader BR = new BufferedReader(new FileReader(wordsFile));
		HashMap<Integer, Integer> multiNumTimesHash = new HashMap<Integer, Integer>();

		String currentLine;
		while (null != (currentLine = BR.readLine())) {

			String[] oneLine = currentLine.split("\\s");
			multiNumTimesHash.put(Integer.parseInt(oneLine[0]), Integer.parseInt(oneLine[2]));
//			System.out.println(oneLine[0]+" "+oneLine[2]);
		}

		BR.close();
		return multiNumTimesHash;
	}

	/**
	 * read from file to get the following information token, corresponding
	 * Integer, and how many times this token appears in training data.
	 *
	 * @param fileName
	 * @return HashMap<String, int[]>, String is the token, int[0] is the
	 * corresponding Integer, int[1] is the times this token appears.
	 * @throws NumberFormatException
	 * @throws IOException
	 */
	public static HashMap<String, int[]> getMultiWordNumTimesHash(String fileName) throws NumberFormatException, IOException {
		File wordsFile = new File(fileName);

		BufferedReader BR = new BufferedReader(new FileReader(wordsFile));
		HashMap<String, int[]> multiWordNumTimesHash = new HashMap<String, int[]>();

		String currentLine;
		while (null != (currentLine = BR.readLine())) {

			String[] oneLine = currentLine.split("\\s");//String[] oneLine=currentLine.split("\\s+");
			int[] numTimes = new int[2];
			numTimes[0] = Integer.parseInt(oneLine[0]);
			numTimes[1] = Integer.parseInt(oneLine[2]);
			multiWordNumTimesHash.put(oneLine[1], numTimes);
//			System.out.println(oneLine[0]+" "+oneLine[2])
		}

		BR.close();
		return multiWordNumTimesHash;
	}
//	public static void main(String[] args) throws IOException
//	{
//		getMultiWordsHash geter=new getMultiWordsHash();
//		geter.getWordsNumHash("/home/students/huiqin/gutenberg/fordice/target.countsingle");
//		geter.getMultiNumTimesHash("/home/students/huiqin/gutenberg/fordice/target.countmulti");
//	}
}
