/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix;

import java.io.IOException;
import java.util.HashMap;
import java.util.Set;

/**
 * get the dice value of a sentence pair
 *
 * @author Huiqin Körkel-Qu
 */
public class sentencePairDice {

	wordPairDice WordPairDice;

	public sentencePairDice(wordPairDice WordPairDice) {
		this.WordPairDice = WordPairDice;
	}

	public double get(int[] sourceSent, int[] targetSent) throws IOException {

		if (null == sourceSent || null == targetSent || sourceSent.length == 0 || targetSent.length == 0) {
			return 0.0;
		}

		/*
		 * sort the words in sentence into tokens and how many times the token
		 * appears
		 *
		 * e.g setence 1 3 1 4 will change to be <1,3>, <3,1> , <4,1>
		 */
		HashMap<Integer, Integer> sourceTokenTimes = new HashMap<Integer, Integer>();
		HashMap<Integer, Integer> targetTokenTimes = new HashMap<Integer, Integer>();

		for (int i = 0; i < sourceSent.length; i++) {

			if (null == (sourceTokenTimes.get(sourceSent[i]))) {
				sourceTokenTimes.put(sourceSent[i], 1);

			} else {
				int times = sourceTokenTimes.get(sourceSent[i]) + 1;
				sourceTokenTimes.put(sourceSent[i], times);
			}
		}

		for (int i = 0; i < targetSent.length; i++) {

			if (null == (targetTokenTimes.get(targetSent[i]))) {
				targetTokenTimes.put(targetSent[i], 1);

			} else {
				int times = targetTokenTimes.get(targetSent[i]) + 1;
				targetTokenTimes.put(targetSent[i], times);
			}
		}

		double sentDiceValue = 0;
		// if there are source and target are not used for dice value,
		//continue to search for more dice increasement.
		while (sourceTokenTimes.size() > 0 && targetTokenTimes.size() > 0) {
			Set<Integer> sourceTokenKeys = sourceTokenTimes.keySet();
			Set<Integer> targetTokenKeys = targetTokenTimes.keySet();

			double maxDiceValue = 0.0;
			int maxSource = -1;
			int maxTarget = -1;
			for (int source : sourceTokenKeys) {
				for (int target : targetTokenKeys) {
					double newDiceValue = WordPairDice.get(source, target);
//					System.out.println("dice for "+ source + " and "+target +" is: "+newDiceValue);
					//take the token pair that has the highest dice value,
					if (newDiceValue > maxDiceValue) {
						maxSource = source;
						maxTarget = target;
						maxDiceValue = newDiceValue;
					}
				}
			}
			// the max dice value of word pair is 0. jump out of the loop;

			if (-1 == maxSource || -1 == maxTarget) {
				break;
			}

//			System.out.println("max source word and target word are "+ maxSource+" and "+maxTarget);
			//  remove the words used for this dice value(deduce the times that the token is not used).
			int sourceTimes = sourceTokenTimes.get(maxSource);
			int targetTimes = targetTokenTimes.get(maxTarget);

			int maxDiceTimes = Math.min(sourceTimes, targetTimes);

			int newSourceTimes = sourceTimes - maxDiceTimes;
			int newTargetTimes = targetTimes - maxDiceTimes;
			if (0 < newSourceTimes) {
				sourceTokenTimes.put(maxSource, newSourceTimes);
			} else {
				sourceTokenTimes.remove(maxSource);
			}

			if (0 < newTargetTimes) {
				targetTokenTimes.put(maxTarget, newTargetTimes);
			} else {
				targetTokenTimes.remove(maxTarget);
			}

			sentDiceValue += maxDiceValue * maxDiceTimes;
		}

		return sentDiceValue * 2.0 / (sourceSent.length + targetSent.length);
	}
}