/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix.train;

import java.util.Iterator;
import java.util.Map;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import diceMatrix.getStopWords;

/**
 * count how many times a token appear in a corpus(source or target)
 *
 * @author huiqin
 */
public class wordCount {

	HashMap<String, Integer> stopWordsList = new HashMap<String, Integer>();
	// the 
	HashMap<String, Integer> allWordsHash = new HashMap<String, Integer>();
	String outputFile;
	HashMap<String, int[]> multiWordsTimesHash;
	String[] singleWords;

	public void count(File file, String stopWordFile, String outputFile) throws IOException {
		//System.out.println("count file "+file.getAbsolutePath());
		getStopWords.getList(stopWordFile, stopWordsList);
		this.outputFile = outputFile;

		int rareWordsNum = 0;

		if (null == stopWordsList || stopWordsList.size() == 0) {

			FileReader textReader = new FileReader(file);
			BufferedReader textBR = new BufferedReader(textReader);
			String currentLine;
			while (null != (currentLine = textBR.readLine())) {
				String[] wordsList = currentLine.split("\\s+");
				for (int j = 0; j < wordsList.length; j++) {

					//int index=WordsList.indexOf(wordsList[j]);

					//if the word has already appeared before
					if (null != allWordsHash.get(wordsList[j])) {
						int index = allWordsHash.get(wordsList[j]);
						allWordsHash.put(wordsList[j], index + 1);
					} // if not, add it to the twords list.
					else if (allWordsHash.size() <= 200000) {
						allWordsHash.put(wordsList[j], 1);

					} else {
						rareWordsNum += 1;
					}
				}
			}
			textBR.close();
			textReader.close();

		} //stopwordlist is not null
		else {

			// System.out.println("the words in "+i +" file are being counted\n");
			FileReader textReader = new FileReader(file);
			BufferedReader textBR = new BufferedReader(textReader);
			String currentLine;
			while (null != (currentLine = textBR.readLine())) {
				String[] wordsList = currentLine.split("\\s+");
				for (int j = 0; j < wordsList.length; j++) {
					//if the word is stopword, skip
					if (null != stopWordsList.get(wordsList[j])) {
						continue;
					}

					//if the word has already appeared before
					if (null != allWordsHash.get(wordsList[j])) {
						int currentNum = allWordsHash.get(wordsList[j]);
						allWordsHash.put(wordsList[j], currentNum + 1);

					} // if not, add it to the untwords list.
					else if (allWordsHash.size() <= 200000) {
						allWordsHash.put(wordsList[j], 1);

					} else {
						rareWordsNum += 1;
					}
				}
			}
			textBR.close();
			textReader.close();
		}

//	    int WordsNum=allWordsHash.size()+1;
		// ArrayList<String> WordsList=new ArrayList<String>;
		//ArrayList<Integer> WordsNum;
	}

	//if a word appear only ones, save it into singleWords, to save place.
	// and sort the single word according to alphabet, sort the multi words according to the times it appear.
	//remove allWordsHash, since all the information of it moves to singleWords, and multiWordsTimesHash.
	@SuppressWarnings("unchecked")
	public void cutSingleWords() throws IOException {
		HashMap<String, Integer> multiWordsHash = new HashMap<String, Integer>();
		Iterator iter = allWordsHash.entrySet().iterator();
		ArrayList singleWordsList = new ArrayList<String>(40000);
		while (iter.hasNext()) {
			Map.Entry entry = (Map.Entry) iter.next();
			String word = (String) entry.getKey();
			int times = (Integer) entry.getValue();
			if (times == 1) {
				singleWordsList.add(word);
			} else {
				multiWordsHash.put(word, times);
			}
		}
		singleWordsList.trimToSize();
		singleWords = new String[singleWordsList.size()];
		for (int i = 0; i < singleWordsList.size(); i++) {
			singleWords[i] = singleWordsList.get(i).toString();
		}
		//System.out.println("there are "+singleWords.length+ " single words");
		//System.out.println("there are "+multiWordsHash.size()+ " muliti words");
		Sort.quickString(singleWords);
		allWordsHash.clear();
		this.sortMultiWordsTimesHash(multiWordsHash);
		this.outPutSingle(singleWords, outputFile);
		multiWordsHash.clear();
	}

	@SuppressWarnings("unchecked")
	public void sortMultiWordsTimesHash(HashMap<String, Integer> WordsHash) throws IOException {
		for (int i = 0; i < WordsHash.size(); i++) {
		}
		String WordsList[] = new String[(WordsHash.size())];
		int WordsNum[] = new int[(WordsHash.size())];

		Iterator iter = WordsHash.entrySet().iterator();
		int wordOrder = 0;
		while (iter.hasNext()) {
			Map.Entry m = (Map.Entry) iter.next();
			String word = m.getKey().toString();
			int times = (Integer) m.getValue();
			WordsList[wordOrder] = word;
			WordsNum[wordOrder] = times;
			wordOrder += 1;
		}
		//System.out.println("there are "+WordsNum.length +"  words");

		Sort.quickIntString(WordsNum, WordsList);

		multiWordsTimesHash = new HashMap<String, int[]>();
		for (int i = 0; i < WordsNum.length; i++) {
			int timeNum[] = new int[2];
			timeNum[0] = i;
			timeNum[1] = WordsNum[i];
			multiWordsTimesHash.put(WordsList[i], timeNum);
		}

		this.outPutMulti(WordsNum, WordsList, outputFile);
	}

	public boolean outPutMulti(int[] wordsNumList, String[] wordsList, String outputFile) throws IOException {

		if (wordsNumList.length != wordsList.length) {
			System.out.println("the number of words and the number of appear times are not equal!");
			return false;
		}

		File countFile = new File(outputFile + "multi");

		FileWriter countWriter = new FileWriter(countFile);
		BufferedWriter countBW = new BufferedWriter(countWriter);

		for (int i = 0; i < wordsList.length; i++) {
			StringBuilder newLine = new StringBuilder();
			newLine.append(i + " ").append(wordsList[i]).append(" ").append(wordsNumList[i]);
			countBW.write(newLine.toString());
			countBW.write("\n");
		}

		countBW.flush();
		countBW.close();
		countWriter.close();
		return true;
	}

	public boolean outPutSingle(String[] singleWords, String outputFile) throws IOException {
		File countFile = new File(outputFile + "single");
		FileWriter countWriter = new FileWriter(countFile);

		BufferedWriter countBW = new BufferedWriter(countWriter);
		int multiWordsNum = multiWordsTimesHash.size();
		for (int i = 0; i < singleWords.length; i++) {
			countBW.write((i + multiWordsNum) + "");
			countBW.write(" " + singleWords[i]);

			countBW.write("\n");
		}

		countBW.flush();
		countBW.close();
		countWriter.close();
		return true;
	}

	@SuppressWarnings("unchecked")
	public HashMap<Integer, Integer> getNumTimesHash(HashMap<String, int[]> wordsListNumHash) {
		HashMap<Integer, Integer> numTimesHash = new HashMap<Integer, Integer>();
//	 int singleWordOrder=0;
		Iterator iter = wordsListNumHash.entrySet().iterator();

		while (iter.hasNext()) {
			Map.Entry m = (Map.Entry) iter.next();
			int[] numTimes = (int[]) m.getValue();
			int num = numTimes[0];
			int times = numTimes[1];

			numTimesHash.put(num, times);
		}

		return numTimesHash;
	}

	/**
	 *
	 * quick sort modified from internet code
	 *
	 */
	static class Sort {

		public static void quickIntString(int[] number, String[] wordsOrder) {
			sortIntString(number, wordsOrder, 0, number.length - 1);
		}

		public static void quickString(String[] singleWords) {
			sortString(singleWords, 0, singleWords.length - 1);
		}

		private static void sortIntString(int[] number, String[] wordsOrder, int left, int right) {
			if (left < right) {
				int i = left;
				int j = right + 1;
				while (true) {

					while (i + 1 < number.length && number[++i] > number[left]) ;

					while (j - 1 > -1 && number[--j] < number[left]) ;
					if (i >= j) {
						break;
					}
					swap(number, i, j);
					swap(wordsOrder, i, j);
				}
				swap(number, left, j);
				swap(wordsOrder, left, j);
				sortIntString(number, wordsOrder, left, j - 1);
				sortIntString(number, wordsOrder, j + 1, right);
			}
		}

		private static void swap(int[] number, int i, int j) {
			int t = number[i];
			number[i] = number[j];
			number[j] = t;
		}

		private static void swap(String[] wordsOrder, int i, int j) {
			String t = wordsOrder[i];
			wordsOrder[i] = wordsOrder[j];
			wordsOrder[j] = t;
		}
	}

	private static void sortString(String a[], int lo0, int hi0) {
		int lo = lo0;
		int hi = hi0;
		String mid;
		if (hi0 > lo0) {
			mid = a[(lo0 + hi0) / 2];

			while (lo <= hi) {
				while ((lo < hi0) && (a[lo].compareTo(mid) < 0)) {
					++lo;
				}
				while ((hi > lo0) && (a[hi].compareTo(mid) > 0)) {
					--hi;
				}
				if (lo <= hi) {
					String t = a[hi];
					a[hi] = a[lo];
					a[lo] = t;
					++lo;
					--hi;
				}
			}
			if (lo0 < hi) {
				sortString(a, lo0, hi);
			}
			if (lo < hi0) {
				sortString(a, lo, hi0);
			}
		}
	}
}