/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * given a pair of word, return the dice value of this word pair.
 *
 * @author Huiqin Körkel-Qu
 */
public class wordPairDice {

	HashMap<Integer, Integer> sourceMultiHash;
	HashMap<Integer, Integer> targetMultiHash;
	int[][] diceMatrix;
	ArrayList<HashMap<Integer, Integer>> commenArray;
	String rareFilePath;

	/**
	 *
	 * @param sourceMultiHash: to get how many times the source word appears
	 * @param targetMultiHash: to get how many times the target word appears
	 * @param diceMatrix: to get how many times this word pair appears
	 * @param commenArray: to get how many times this word pair appears
	 * @param rareFilePath: not used anymore, should be removed.
	 */
	public wordPairDice(HashMap<Integer, Integer> sourceMultiHash, HashMap<Integer, Integer> targetMultiHash,
		int[][] diceMatrix, ArrayList<HashMap<Integer, Integer>> commenArray, String rareFilePath) {

		this.sourceMultiHash = sourceMultiHash;
		this.targetMultiHash = targetMultiHash;
		this.diceMatrix = diceMatrix;
		this.commenArray = commenArray;
		this.rareFilePath = rareFilePath;
	}

	public double get(int sourceWord, int targetWord) throws IOException {
		int sourceAppearTimes;
		int targetAppearTimes;
//		int wordPairAppearTimes;
		int bothAppearTimes;
		//source or target never appeared in training 
		if (sourceWord < 0 || targetWord < 0) {
			return 0.0;
		}

		if (null != sourceMultiHash.get(sourceWord)) {
			sourceAppearTimes = sourceMultiHash.get(sourceWord);

		} else {
			sourceAppearTimes = 1;
		}

		if (null != targetMultiHash.get(targetWord)) {
			targetAppearTimes = targetMultiHash.get(targetWord);

		} else {
			targetAppearTimes = 1;
		}

		bothAppearTimes = sourceAppearTimes + targetAppearTimes;

		double diceValue = 0.0;
//		System.out.println(sourceWord+";"+targetWord);
//		System.out.println("source and target appear times are "+sourceAppearTimes+" and "+targetAppearTimes);
		//both are often word, find the pair appearance times in dice matrix

		if (sourceWord < myConst.maxSM) {
			if (targetWord < myConst.maxTM) {

				diceValue = diceMatrix[sourceWord][targetWord] * 2.0 / bothAppearTimes;

			} else {
//				System.out.println("source is often word, target is  commen");
				HashMap<Integer, Integer> targetHash = commenArray.get(sourceWord);

				if (null != targetHash.get(targetWord)) {
//					System.out.println(" dice value is "+(targetHash.get(targetWord)*2.0/bothAppearTimes));
					diceValue = (targetHash.get(targetWord) * 2.0 / bothAppearTimes);
				}
			}
		} //source is commen word, target is often or commen, find the word pair appearance times in commenArray.
		else {
//			System.out.println("source is commen word, target is often or commen");
			HashMap<Integer, Integer> targetHash = commenArray.get(sourceWord);

			if (null != targetHash.get(targetWord)) {
//				System.out.println(" dice value is "+(targetHash.get(targetWord)*2.0/bothAppearTimes));
				diceValue = (targetHash.get(targetWord) * 2.0 / bothAppearTimes);
			}
		}
//		System.out.println("dice value is "+diceValue);
		return diceValue;
	}
}
