/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package general.com.posMap;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Hashtable;

import diceMatrix.myConst;

/**
 * to create the POS (part-of-speech) map so that the sentence pair's POS
 * agreement can be calculated.
 *
 * @author Huiqin Körkel-Qu
 */
public class posHashtable {

	public static void main(String[] args) {
		posHashtable p = new posHashtable();
		Hashtable<String, ArrayList<String>> t = p.createPosMapTable("de", "en", "posMapDeEnPure");
		System.out.println("t: " + t);
	}
//	the file of posHash is saved in the directory of src.
	String posHashPath;

	public Hashtable<String, ArrayList<String>> createPosMapTable(String language1, String language2, String posMapFileName) {
		String path = new general.getHelpPath().get();
		posHashPath = path + myConst.helpFilesPath;

		//in this programm, language1 is de, language is en. 
		//it can be extended to othere languages too. 
		String sLanguage = language1.toLowerCase();
		String tLanguage = language2.toLowerCase();
		File posMapFile = new File(posHashPath + "/" + posMapFileName);
		System.out.println("the POS map file is " + posMapFile.getAbsolutePath());

		if (!posMapFile.isFile()) {
			System.out.println("the POS map file " + posMapFile.getAbsolutePath() + "doesn't exist!");
			System.out.println("the POS is set to be " + posMapFileName);
			return null;
		}

		Hashtable<String, ArrayList<String>> posTable = new Hashtable<String, ArrayList<String>>();

		if (sLanguage.equals("de")
			&& tLanguage.toLowerCase().equals("en")) {
			try {

				FileReader posMapReader = new FileReader(this.posHashPath + "/" + posMapFileName);
				BufferedReader br = new BufferedReader(posMapReader);
				String currentLine;
				String[] stL = new String[2];
				currentLine = br.readLine();
				stL[0] = currentLine.split("\t")[0].trim();
				stL[1] = currentLine.split("\t")[1].trim();
				if (!stL[0].toLowerCase().equals("de") || !stL[1].toLowerCase().endsWith("en")) {
					System.out.println("lanuage error in posMapFile");
					return null;
				}

				//every line saves one kind of POS map between the two languages.
				while (null != (currentLine = br.readLine())) {
					String sPos, tPos;
					sPos = currentLine.split("\t")[0].trim();
					try {
						tPos = currentLine.split("\t")[1].trim();
						ArrayList<String> tList = new ArrayList<String>();
						for (int i = 0; i < tPos.split(" ").length; i++) {
							tList.add(tPos.split(" ")[i].trim());
						}
						posTable.put(sPos, tList);
					} catch (Exception o) {
						ArrayList<String> tList = new ArrayList<String>();
						tList.add("");
						posTable.put(sPos, tList);
					}
				}

				br.close();
				posMapReader.close();
			} catch (Exception o) {
				o.printStackTrace();
			}
			return posTable;

		} else if (sLanguage.equals("en") && tLanguage.equals("de")) {
			try {
				FileReader posMapReader = new FileReader(posMapFileName);
				BufferedReader br = new BufferedReader(posMapReader);
				String currentLine;
				String[] stL = new String[2];
				currentLine = br.readLine();
				stL[0] = currentLine.split("\t")[0].trim();
				stL[1] = currentLine.split("\t")[1].trim();
				if (!stL[0].toLowerCase().equals("en") || !stL[1].toLowerCase().endsWith("de")) {
					System.out.println("lanuage error in posMapFile");
					return null;
				}

				while (null != (currentLine = br.readLine())) {
					String sPos, tPos;
					sPos = currentLine.split("\t")[0].trim();
					try {
						tPos = currentLine.split("\t")[1].trim();
						ArrayList<String> tList = new ArrayList<String>();
						for (int i = 0; i < tPos.split(" ").length; i++) {
							tList.add(tPos.split(" ")[i].trim());
						}
						posTable.put(sPos, tList);
					} catch (Exception o) {
						ArrayList<String> tList = new ArrayList<String>();
						tList.add("");
						posTable.put(sPos, tList);
					}
				}

				br.close();
				posMapReader.close();
			} catch (Exception o) {
				o.printStackTrace();
			}

			return posTable;

		} else {
			System.out.println("no hashtable for this pair source and target languages yet! ");
			return null;
		}
	}
}
