/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package general.orderAlign;

import java.util.*;

/**
 * compare according to sIds, if sIds are the same, compare according to tIds.
 * 
 * @author Huiqin Körkel-Qu
 */
public class Mycomparator implements Comparator<Object> {

	/**
	 * compare according to sIds, if sIds are the same, compare according to
	 * tIds.
	 *
	 * @author Huiqin Körkel-Qu
	 */
	public int compare(Object S1, Object S2) {
		int[] ids1 = getIds(S1);
		int[] ids2 = getIds(S2);
		Integer S1SId = ids1[0];
		Integer S2SId = ids2[0];
		int flag = S1SId.compareTo(S2SId);
		if (flag == 0 || S1SId == 0 || S2SId == 0) {
			Integer S1TId = ids1[1];
			Integer S2TId = ids2[1];
			return S1TId.compareTo(S2TId);
		} else {
			return flag;
		}
	}

	private static int[] getIds(Object S) {
		int sId;
		int tId;
		int[] stId = new int[2];
		String input = S.toString();

		String sPart = input.split(";")[0].split("\\s+")[0];
		if (sPart.equals("")) {
			sId = 0;
		} else {
			sId = Integer.parseInt(sPart);
		}
		if (input.split(";").length == 1) {
			tId = 0;
		} else {
			tId = Integer.parseInt(input.split(";")[1].split("\\s+")[0]);
		}
		stId[0] = sId;
		stId[1] = tId;
		return stId;

	}
}
