/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package general.orderAlign;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

/**
 * according to the crf label result, get the alignment eg, there is a line in
 * crf ...... i;j T2 the alignment is i;j-1,j
 * 
 * @author Huiqin Körkel-Qu
 */
public class labelToAlign {

	int featureNum = diceMatrix.myConst.featureNum;

	public static ArrayList<String> extractIndexLabel(String fileName) throws IOException {
		File input = new File(fileName);
		if (!input.isFile()) {
			System.out.println("input file: " + fileName + " doesn't exist!");
			return null;
		}

		ArrayList<String> alignArray = new ArrayList<String>();

		FileReader labelReader = new FileReader(input);
		BufferedReader labelBr = new BufferedReader(labelReader);

		String currentLine;
		while (null != (currentLine = labelBr.readLine())) {
			int lineColumn = currentLine.split("\\s+").length;

			try {
				if (lineColumn < diceMatrix.myConst.featureNum) {
					continue;
				}

				if (!currentLine.split("\\s+")[lineColumn - 1].contains("T")
					&& !currentLine.split("\\s+")[lineColumn - 1].contains("F")
					&& !currentLine.split("\\s+")[lineColumn - 1].contains("S")) {
					continue;
				}
				String currentLabel = currentLine.split("\\s+")[lineColumn - 1];
				String currentIndex = currentLine.split("\\s+")[lineColumn - 2];

				String currentSource = currentIndex.split(";")[0];
				String currentTarget = currentIndex.split(";")[1];

				if (currentLabel.contains("T")
					&& !currentLabel.contains("T2")
					&& !currentLabel.contains("T3")) {
					alignArray.add(currentIndex);

				} else if (currentLabel.contains("S2")) {
					int sourceNeibour2 = Integer.parseInt(currentSource) - 1;
					if (sourceNeibour2 > 0) {
						alignArray.add(sourceNeibour2 + " " + currentSource + ";" + currentTarget);
					}
				} else if (currentLabel.contains("S3")) {
					int sourceNeibour2 = Integer.parseInt(currentSource) - 1;
					int sourceNeibour3 = Integer.parseInt(currentSource) - 2;
					if (sourceNeibour3 > 0) {
						alignArray.add(sourceNeibour3 + " "
							+ sourceNeibour2 + " " + currentSource + ";" + currentTarget);
					}
					
				} else if (currentLabel.contains("T2")) {
					int targetNeibour2 = Integer.parseInt(currentTarget) - 1;
					if (targetNeibour2 > 0) {
						alignArray.add(currentSource + ";" + targetNeibour2 + " " + currentTarget);
					}

				} else if (currentLabel.contains("T3")) {
					int targetNeibour2 = Integer.parseInt(currentTarget) - 1;
					int targetNeibour3 = Integer.parseInt(currentTarget) - 2;
					if (targetNeibour3 > 0) {
						alignArray.add(currentTarget + ";" + targetNeibour3 + " "
							+ targetNeibour2 + " " + currentTarget);
					}

				} else if (currentLabel.equals("F")) {
					continue;
				} else {
					System.out.println("unknown label:" + currentLabel + ". this should not happen!");
				}
			} catch (Exception o) {
				System.out.print("the line  ");
				System.out.println(currentLine);
				System.out.println(" is not a correct crf output line!");
			}

		}
		labelBr.close();
		labelReader.close();
		return alignArray;
	}
}