/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package general.orderAlign;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

/**
 * sort the alignment into sequences, and the longer sequences will be regarded
 * as the real alignment if there are conflicts.
 *
 * @author Huiqin Körkel-Qu
 */
public class longSeqSort {

	static int sureSeqLength = 10;
	static int possSeqLength = 5;

	public static ArrayList<String> getLongSeqAlign(ArrayList<ArrayList<String>> allGroups, int seqLength) {
		ArrayList<String> longSeqAlign = new ArrayList<String>();
		for (ArrayList<String> alignSeq : allGroups) {
			if (alignSeq.size() >= seqLength) {
				for (String align : alignSeq) {
					longSeqAlign.add(align);
				}
			}
		}
		return longSeqAlign;
	}

	/**
	 * sort the alignments into sequences( no overlapping, no un-continuous part
	 * )
	 *
	 * @param allAlign
	 * @return
	 * @throws IOException
	 */
	public static ArrayList<ArrayList<String>> getExactGroups(ArrayList<String> allAlign) throws IOException {

		ArrayList<ArrayList<String>> alignGroups = new ArrayList<ArrayList<String>>();

		for (int i = 0; i < allAlign.size(); i++) {

			int currentSStart = Integer.parseInt(allAlign.get(i).split(";")[0].split("\\s+")[0]);
			int currentTStart = Integer.parseInt(allAlign.get(i).split(";")[1].split("\\s+")[0]);
			int currentSEnd = Integer.parseInt(allAlign.get(i).split(";")[0].split("\\s+")[allAlign.get(i).split(";")[0].split("\\s+").length - 1]);
			int currentTEnd = Integer.parseInt(allAlign.get(i).split(";")[1].split("\\s+")[allAlign.get(i).split(";")[1].split("\\s+").length - 1]);
			String currentAlign = allAlign.get(i);

			int findGroup = -1;
			int bestGroup = -1;

//			to check every align group to see which group the current alignment belong to.
			for (int j = 0; j < alignGroups.size(); j++) {

				// the alignment is already there.
				for (String align : alignGroups.get(j)) {
					if (align.trim().equals(currentAlign.trim())) {
						findGroup = 3;
						break;
					}
				}

				if (findGroup == 3) {
					break;
				}

//				to get the first and last alignments in every group, so that we can compare it with the current alignment
//				to see if it is continues from it.
				int currentGroupSize = alignGroups.get(j).size();
				String lastAlignment = alignGroups.get(j).get(currentGroupSize - 1);
				String firstAlignment = alignGroups.get(j).get(0);

				int lastS, lastT;
				int firstS, firstT;

				String lastSString = lastAlignment.split(";")[0];
				String lastTString = lastAlignment.split(";")[1];

				String firstSString = firstAlignment.split(";")[0];
				String firstTString = firstAlignment.split(";")[1];

				lastS = Integer.parseInt(lastSString.split("\\s+")[lastSString.split("\\s+").length - 1]);
				lastT = Integer.parseInt(lastTString.split("\\s+")[lastTString.split("\\s+").length - 1]);

				firstS = Integer.parseInt(firstSString.split("\\s+")[0]);
				firstT = Integer.parseInt(firstTString.split("\\s+")[0]);

				// exact continuous at the end sentence alignment seq, the best continues, stop
				if (lastS == currentSStart - 1 && lastT == currentTStart - 1) {
					bestGroup = j;
					alignGroups.get(bestGroup).add(currentAlign);
					findGroup = 1;
					break;
				} //exact continuous from  the beginning
				else if (firstS == currentSEnd + 1 && firstT == currentTEnd + 1) {
					bestGroup = j;
					findGroup = 2;
					alignGroups.get(bestGroup).add(0, currentAlign);
					break;
				} //smaller overlapping at end from both source and target side.
				// if the align is 31:20
				// the align 31;21 will not be continuous, but another seq!
				else if (currentSStart <= lastS && currentTStart <= lastT && (currentSStart >= lastS - 2 && currentTStart >= lastT - 2)) {

					bestGroup = j;
					alignGroups.get(bestGroup).add(currentAlign);
					findGroup = 1;
					break;

				} else if (currentSEnd >= firstS && currentTEnd >= firstT && (currentSEnd < firstS - 2 && currentTEnd < firstT - 2)) {

					bestGroup = j;
					alignGroups.get(bestGroup).add(currentAlign);
					findGroup = 1;
					break;

				} else {
				}
			}

			if (findGroup == -1) {
				ArrayList<String> newArray = new ArrayList<String>();
				newArray.add(currentAlign);
				alignGroups.add(newArray);
			}
		}

		ArrayComparator arrayorder = new ArrayComparator();
		Collections.sort(alignGroups, arrayorder);

		return alignGroups;
	}

	/**
	 * insert the seq groupToInsert into the seqs List existGroups according to
	 * the alignment Ids
	 *
	 * @param existGroups
	 * @param groupToInsert
	 * @return
	 */
	public static int insertList(ArrayList<ArrayList<String>> existGroups, ArrayList<String> groupToInsert) {
		int order = -1;

		if (null == existGroups) {
			order = 0;
			return order;
		}
		if (groupToInsert.size() < 2) {
			return -1;
		}

		// to see if the group should be insert at the beginning
		int insertStartS;
		int insertStartT;
		int insertEndS;
		int insertEndT;
		int existStartS;
		int existStartT;
		int existEndT;
		int existEndS;

		{
			String existStart = existGroups.get(0).get(0);
			existStartS = Integer.parseInt(existStart.split(";")[0].split("\\s+")[existStart.split(";")[0].split("\\s+").length - 1]);
			existStartT = Integer.parseInt(existStart.split(";")[1].split("\\s+")[existStart.split(";")[1].split("\\s+").length - 1]);

			String insertEnd = groupToInsert.get(groupToInsert.size() - 1);
			insertEndS = Integer.parseInt(insertEnd.split(";")[0].split("\\s+")[insertEnd.split(";")[0].split("\\s+").length - 1]);
			insertEndT = Integer.parseInt(insertEnd.split(";")[1].split("\\s+")[insertEnd.split(";")[1].split("\\s+").length - 1]);


			if (insertEndS < existStartS && insertEndT < existStartT) {
				order = 0;
				return order;
			} else if (Math.abs(insertEndS - existStartS) <= 2
				&& Math.abs(insertEndT - existStartT) <= 2
				&& groupToInsert.size() > possSeqLength) //			    else if ((insertEndS-2<existStartS&&existStartS<insertEndS)&&
			//			    		(insertEndT-2<existStartT&&insertEndT>existStartT)&&groupToInsert.size()>possSeqLength)
			{
				order = 0;
				return order;
			} else {
			}
		}

		String insertStart = groupToInsert.get(0);
		String insertEnd = groupToInsert.get(groupToInsert.size() - 1);

		for (int i = 0; i < existGroups.size() - 1; i++) {
			//the end of last group
			String existEnd = existGroups.get(i).get(existGroups.get(i).size() - 1);

			// the start of next group	
			String existStart = existGroups.get(i + 1).get(0);
			// to see if the group to insert is between .
			existEndS = Integer.parseInt(existEnd.split(";")[0].split("\\s+")[existEnd.split(";")[0].split("\\s+").length - 1]);
			existEndT = Integer.parseInt(existEnd.split(";")[1].split("\\s+")[existEnd.split(";")[1].split("\\s+").length - 1]);

			existStartS = Integer.parseInt(existStart.split(";")[0].split("\\s+")[existStart.split(";")[0].split("\\s+").length - 1]);
			existStartT = Integer.parseInt(existStart.split(";")[1].split("\\s+")[existStart.split(";")[1].split("\\s+").length - 1]);

			insertEndS = Integer.parseInt(insertEnd.split(";")[0].split("\\s+")[insertEnd.split(";")[0].split("\\s+").length - 1]);
			insertEndT = Integer.parseInt(insertEnd.split(";")[1].split("\\s+")[insertEnd.split(";")[1].split("\\s+").length - 1]);

			insertStartS = Integer.parseInt(insertStart.split(";")[0].split("\\s+")[insertStart.split(";")[0].split("\\s+").length - 1]);
			insertStartT = Integer.parseInt(insertStart.split(";")[1].split("\\s+")[insertStart.split(";")[1].split("\\s+").length - 1]);
//				if(insertStart.contains("255;255 256"))
//				{
//					System.out.println("the last seq end is :"+existEndS+";"+existEndT);
//					System.out.println("the next seq begin is :"+existStartS+";"+existStartT);
//					System.out.println("insert start is : "+insertStartS+";"+insertStartT);
//					System.out.println("insert end is : "+insertEndS+";"+insertEndT);
//				}
			//between two groups
			if (insertStartS > existEndS && insertStartT > existEndT
				&& insertEndS < existStartS && insertEndT < existStartT) {
				order = i + 1;

				return order;
			} //				if the group is sure, small overlapping is allowed!
			//ovlapping appears with the   seq above 
			else if (groupToInsert.size() > possSeqLength
				&& Math.abs(insertStartS - existEndS) <= 2 && Math.abs(existEndT - insertStartT) <= 2
				&& (insertEndS < existStartS && insertEndT < existStartT)) //				
			{
				order = i + 1;
				return order;
			} //ovlapping appears with the   seq below 
			else if (groupToInsert.size() > possSeqLength
				&& Math.abs(insertEndS - existStartS) <= 2 && Math.abs(insertEndT - existStartT) <= 2
				&& (insertStartS > existEndS && insertStartT > existEndT)) {
				order = i + 1;
				return order;
			} //ovlapping appears with both 
			else if (groupToInsert.size() > possSeqLength + 2
				&& Math.abs(insertEndS - existStartS) <= 2 && Math.abs(existStartT - insertEndT) <= 2
				&& Math.abs(insertStartS - existEndS) <= 2 && Math.abs(existEndT - insertStartT) <= 2) {
				order = i + 1;
				return order;
			} else {
			}
		}

		// to insert at the end
		ArrayList<String> lastExistGroup = existGroups.get(existGroups.size() - 1);
		String existEnd = lastExistGroup.get(lastExistGroup.size() - 1);

		existEndS = Integer.parseInt(existEnd.split(";")[0].split("\\s+")[existEnd.split(";")[0].split("\\s+").length - 1]);
		existEndT = Integer.parseInt(existEnd.split(";")[1].split("\\s+")[existEnd.split(";")[1].split("\\s+").length - 1]);

		insertStartS = Integer.parseInt(insertStart.split(";")[0].split("\\s+")[insertStart.split(";")[0].split("\\s+").length - 1]);
		insertStartT = Integer.parseInt(insertStart.split(";")[1].split("\\s+")[insertStart.split(";")[1].split("\\s+").length - 1]);

		if (insertStartS > existEndS && insertStartT > existEndT) {
			order = existGroups.size();
			return order;
		} //if the group is sure, small overlapping is allowed!
		else if (groupToInsert.size() > possSeqLength
			&& Math.abs(insertStartS - existEndS) <= 2 && Math.abs(insertStartT - existEndT) <= 2) {
			order = existGroups.size();
			return order;
		} else {
			order = -1;
//				if(groupToInsert.size()>2)
//					System.out.println("the group start with "+insertStart+" is throw away");
//				System.out.println("the group start with "+insertStart +" is not merged in the end");   
			return order;
		}
	}

	/**
	 * merge the seqs set in to a seqs List, whose order is decided by the
	 * alignments Ids in seqs
	 *
	 * @param alignGroups
	 * @return
	 */
	public static ArrayList<ArrayList<String>> groupMerge(ArrayList<ArrayList<String>> alignGroups) {

		ArrayComparator arrayorder = new ArrayComparator();
		Collections.sort(alignGroups, arrayorder);
		ArrayList<ArrayList<String>> existGroups = new ArrayList<ArrayList<String>>();
		if (alignGroups.size() <= 1) {

			return existGroups;
		}
		existGroups.add(alignGroups.get(0));


		for (int i = 1; i < alignGroups.size(); i++) {
			int order = insertList(existGroups, alignGroups.get(i));
//			if(i<50)
//			{
//				System.out.println();
//				System.out.println("the current group start with "+alignGroups.get(i).get(0));
//				System.out.println("the current group end with "+alignGroups.get(i).get(alignGroups.get(i).size()-1));
//			}

			if (-1 != order) {
//				if(order==0)
//				{
//					System.out.println("the group insert at the begining ");
//					
//				}
//				else if(existGroups.size()>order&&order>0)
//				{
//					System.out.println("the group before insert place end with" +existGroups.get(order-1).get(existGroups.get(order-1).size()-1));
//					System.out.println("the group after insert place start with " +existGroups.get(order).get(0));
//				}
//					
//				
//					
//			
//				else if(existGroups.size()<=order)
//				System.out.println("the group insert at the end");		

				existGroups.add(order, alignGroups.get(i));
			} else {
//				if(i<50)
//				System.out.println("the current group will not be inserted!");
//				
			}

		}

		return existGroups;
	}

	/**
	 * from the alignment result of crf to the last alignment(seqs selection,
	 * add 1:0 and 0:1 alignment)
	 *
	 * @param allAlign
	 * @return
	 * @throws IOException
	 */
	public static ArrayList<String> arrayToAligns(ArrayList<String> allAlign) throws IOException {
		ArrayList<String> lastAlign = new ArrayList<String>();

		Mycomparator alignorder = new Mycomparator();
		Collections.sort(allAlign, alignorder);
		ArrayList<ArrayList<String>> alignGroups = longSeqSort.getExactGroups(allAlign);
		ArrayList<ArrayList<String>> keptAlign = longSeqSort.groupMerge(alignGroups);
		for (int k = 0; k < keptAlign.size(); k++) {
			for (int j = 0; j < keptAlign.get(k).size(); j++) {
				lastAlign.add(keptAlign.get(k).get(j));
			}
		}
		conflict1to1.resolve(lastAlign);
		conflict2to1.resolve(lastAlign);
		conflict1to2.resolve(lastAlign);
		removeMulti.remove(lastAlign);

		return lastAlign;
	}

	public static void main(String[] args) throws IOException {

		String FILEPATH = "/home/students/huiqin/gutenberg/result";
//	  String FILEPATH=args[0];

		File dir = new File(FILEPATH);
		File file[] = dir.listFiles();
		if (file.length == 0) {
			System.out.println("input empty");
		}

		for (int i = 0; i < file.length; i++) {
//		  if(file[i].isFile()&&file[i].getName().equals("4135819_1of1-4133743_1of1.allAlign"))	
			if (file[i].isFile() && file[i].getName().endsWith("alignment.all")) {

				ArrayList<String> allAlign = new ArrayList<String>();
				allAlign = labelToAlign.extractIndexLabel(file[i].getAbsolutePath());
				BufferedReader br = new BufferedReader(new FileReader(file[i]));
				String currentLine;
				while (null != (currentLine = br.readLine())) {
					if (currentLine.split(";").length == 2) {
						allAlign.add(currentLine);
					}
				}

				Mycomparator alignorder = new Mycomparator();
				Collections.sort(allAlign, alignorder);
				longSeqSort.arrayToAligns(allAlign);

				ArrayList<ArrayList<String>> alignGroups = longSeqSort.getExactGroups(allAlign);

				File alignGroupsFile = new File(file[i].getAbsolutePath() + ".allExactSeqs");
//			   System.out.println(alignGroupsFile.getAbsolutePath());
				BufferedWriter bw = new BufferedWriter(new FileWriter(alignGroupsFile));
				for (int k = 0; k < alignGroups.size(); k++) {
					for (int j = 0; j < alignGroups.get(k).size(); j++) {
						bw.write(alignGroups.get(k).get(j));
						bw.write("\n");
					}
					bw.write("\n");
					bw.flush();
				}

				bw.close();

				ArrayList<ArrayList<String>> keptAlign = longSeqSort.groupMerge(alignGroups);
				File keptAlignFile = new File(file[i].getAbsolutePath() + ".keptSeqs");
//				   System.out.println( keptAlignFile.getAbsolutePath());
				BufferedWriter bwKept = new BufferedWriter(new FileWriter(keptAlignFile));
				for (int k = 0; k < keptAlign.size(); k++) {
					for (int j = 0; j < keptAlign.get(k).size(); j++) {
						bwKept.write(keptAlign.get(k).get(j));
						bwKept.write("\n");
					}
					bwKept.write("\n");
					bwKept.flush();
				}

				bwKept.close();

				ArrayList<String> allLastAlign = new ArrayList<String>();
				for (int k = 0; k < keptAlign.size(); k++) {
					for (int j = 0; j < keptAlign.get(k).size(); j++) {
						allLastAlign.add(keptAlign.get(k).get(j));
					}
				}
				conflict1to1.resolve(allLastAlign);
				conflict2to1.resolve(allLastAlign);
				conflict1to2.resolve(allLastAlign);

//					addEmpty.add(allLastAlign, 10000, 10000, 2);  
				File allLastAlignFile = new File(file[i].getAbsolutePath() + ".allLastAlign");
				BufferedWriter bwAll = new BufferedWriter(new FileWriter(allLastAlignFile));
				for (int k = 0; k < allLastAlign.size(); k++) {

					bwAll.write(allLastAlign.get(k));
					bwAll.write("\n");
				}
				bwAll.close();

//					ArrayList<String> noConAlign=removeMulti.remove(allLastAlign);
//					 File noConflict=new File(file[i].getAbsolutePath().split("\\.")[0]+".noConflict" );
//				   BufferedWriter bwNoC=new BufferedWriter(new FileWriter(noConflict));
//					for(int k=0;k<noConAlign.size();k++)
//					{
//						
//							bwNoC.write(noConAlign.get(k));
//							bwNoC.write("\n");
//				
//					
//					}
//					bwAll.close();

			}
		}
	}
}

/**
 * array comparator according to the array size.
 *
 * @author Huiqin Körkel-Qu
 */
class ArrayComparator implements Comparator<Object> {

	@SuppressWarnings("unchecked")
	public int compare(Object S1, Object S2) {
		Integer array1size = ((ArrayList<ArrayList<String>>) S1).size();
		Integer array2size = ((ArrayList<ArrayList<String>>) S2).size();
		int flag = 0;
		flag = array2size.compareTo(array1size);

		return flag;
	}
}
