/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package generalTest;

import general.writeArray;
import general.com.posMap.posHashtable;
import general.orderAlign.longSeqSort;
import generalTrain.beanPair;
import generalTrain.fileToArray;
import generalTrain.sentenceBean;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;

import diceMatrix.doubleToString;
import diceMatrix.getSent;
import diceMatrix.test.fileDiceTest;

/**
 * beam search for crf label the start area is above and below the diagonal. and
 * check the situation of current alignment to decide if and how to extend the
 * beam area, the area that has been searched will not be searched any more. if
 * source or target has been aligned, the related pairs will also not be
 * searched.
 *
 * if many alignments found are below the diagonal, extend the search area below
 * the diagonal(beamUsedAreaProcent=0.20). if many aignments found are above the
 * diagonal, extend the search area above the diagonal  *
 * if no new good alignment seq (more than 4 continues sentence pair
 * seqLengthLim=5) is found, or enough sentences habe been
 * aligned(alignedNumBorder=0.85)
 *
 * stop the beam search and return the alignments found(may be from many times
 * beam search) as the final alignments
 *
 * @author Huiqin Körkel-Qu
 */
public class beamSearch {
	//if more than alignedNumborder procent source or target has been aligned, beam search ends.

	public static final double alignedNumBorder = 0.90;
	//if the current alignment stays in the beanUsedAreaProcent area, beam search ends.
	public static final double beamUsedAreaProcent = 0.20;
	// only if a seq contains more than 5 sentence pairs, this seq is considered to be a real alignments set.
	// otherwise, it is a candidate alignment seq.
	public static final int seqLengthLim = 3;
	//keep the alignment appear in long seqs, and count the procent of source or target sentences aligned in long seq
	// if this procent is over this value longSeqAlignProcent, and the beam search find no new alignments or alignment seq
	// beam search will not be extended.
	public final double longSeqAlignProcent = 0.60;
	sentenceBean[] allSourceBeans;
	sentenceBean[] allTargetBeans;
	String sourceName;
	String targetName;
	String featureFileName;
	String trainingResultPath;
	String modelName;
	fileToArray sFileArray;
	fileToArray tFileArray;
	int maxSId;
	int maxTId;
	int extendTimes = 0;
	int longSeqLength = 5;// if a seq is longer than 5, we take the alignments and use it for beam extension
	fileDiceTest diceExtra;
	//to mark the area that has been searched.
	int searchedSArea = 0;
	int searchedTArea = 0;
//	ArrayList<ArrayList<String>> candidateAlignSeqs=new ArrayList<ArrayList<String>>();
	ArrayList<String> longSeqAlign = new ArrayList<String>();
	ArrayList<String> allAlign = new ArrayList<String>();
	Hashtable<String, ArrayList<String>> posTable;
	
	/*
	 * beamSearch.
	 */
	public beamSearch(String trainingResultPath, String modelName,
		String sourceName, String targetName, String featureName) throws IOException {

		this.sourceName = sourceName;
		this.targetName = targetName;
		this.featureFileName = featureName;
		this.trainingResultPath = trainingResultPath;
		this.modelName = modelName;

		sFileArray = new fileToArray(this.sourceName);
		tFileArray = new fileToArray(this.targetName);
		// get all the sentence Beans from source and target
		sFileArray.getAllSentenceBeans();
		tFileArray.getAllSentenceBeans();
		this.allSourceBeans = sFileArray.allBeans;
		this.allTargetBeans = tFileArray.allBeans;

		maxSId = this.allSourceBeans.length;
		maxTId = this.allTargetBeans.length;

		String diceSourceName = this.sourceName.split("\\.")[0] + diceMatrix.myConst.extensionForDice;
		String diceTargetName = this.targetName.split("\\.")[0] + diceMatrix.myConst.extensionForDice;
		diceExtra = new fileDiceTest(trainingResultPath, diceSourceName, diceTargetName);

		posHashtable posHash = new posHashtable();
		this.posTable = posHash.createPosMapTable("de", "en", "posMapDeEnPure");
	}

	/**
	 * beam search iteration. according to the alignments found, and the beam
	 * area that has been searched, decide if the beam area should be extended
	 * or not, and get the new beam search area,
	 *
	 * @param existAlign: the alignment has been find before
	 * @param currentSStart: new start of source sentence for the new beam area
	 * @param currentSEnd: the end source sentence of the new beam area
	 * @param currentTStart:new start of target sentence for the new beam area
	 * @param currentTEnd:new start of target sentence for the new beam area
	 * @param modelName: the crf model get from wapiti train. so that the
	 * sentence pairs in the new beam area will be labeled according to this
	 * model.
	 * @return int value, which shows the reason that beam search exists.
	 * @throws NumberFormatException
	 * @throws IOException
	 * @throws InterruptedException
	 */
	public int extendBeam(
		int currentSStart, int currentSEnd, int currentTStart, int currentTEnd, String modelName)
		throws NumberFormatException, IOException, InterruptedException {
		//extendTimes force the beamsearch to end after at most 5 times extending
		extendTimes += 1;
//		System.out.println("beam search interation ");
		if (extendTimes > 1) {
//			this.removeFile(this.featureFileName.split("\\.")[0]+".crfResult");
//		    this.removeFile(this.featureFileName);

			return 1;
		}
		// the cases that the area of this beam has been searched.
		if (searchedSArea > currentSEnd && searchedTArea > currentTEnd) {
			return 1;
		}
		if (searchedSArea > currentSEnd && (currentTEnd - currentTStart) <= 0) {
			return 1;
		}
		if (searchedTArea > currentTEnd && (currentSEnd - currentSStart) <= 0) {
			return 1;
		}

		//the case that the new area is overlapped with searched area
		if (searchedSArea >= currentSStart) {
			currentSStart = searchedSArea;
		}
		if (searchedTArea >= currentTStart) {
			currentTStart = searchedTArea;
		}

		//get the new feature files for the test file
//		ArrayList<ArrayList<String>> currentExistGroups;
		ArrayList<ArrayList<String>> allGroups = longSeqSort.getExactGroups(allAlign);
		longSeqAlign = longSeqSort.getLongSeqAlign(allGroups, longSeqLength);

		ArrayList<ArrayList<String>> diceArrays;

		if (longSeqAlign.size() != 0) {
			//currentExistGroups=conflictDuple.resolve(currentAlign);
			diceArrays =
				this.getDice(longSeqAlign, currentSStart, currentSEnd, currentTStart, currentTEnd);
		} else {
			diceArrays =
				this.getDice(null, currentSStart, currentSEnd, currentTStart, currentTEnd);
		}

		searchedSArea = currentSEnd;
		searchedTArea = currentTEnd;

//	    System.out.println("start to create the feature file for the test files");
		boolean featureSucc = this.getFeatureFile(diceArrays);
//	    System.out.println(" the feature file is created.");
		// feature file is not successfully created.
		if (!featureSucc && null != longSeqAlign) {
			return 7;
		}

		if (!featureSucc && null == longSeqAlign) {
			return 8;
		}

		//crf test(label)
		String crfTestCommand;
//	    System.out.println("start wapiti label process");

		crfTestCommand = "wapiti label -m  " + modelName + "  "
			+ this.featureFileName + " -p  " + this.featureFileName.split("\\.")[0] + ".crfResult";
//	    System.out.println(crfTestCommand);
		Process process = Runtime.getRuntime().exec(crfTestCommand);
		InputStream stderr = process.getErrorStream();
		InputStreamReader isr = new InputStreamReader(stderr);
//        BufferedReader br = new BufferedReader(isr);  
//        String line = null;  
//        while ( (line = br.readLine()) != null)  
//        {
//        	 System.out.println(line); 
//        }

		process.waitFor();

		//get new additional alignment from the crf result
		File crfFile = new File(this.featureFileName.split("\\.")[0] + ".crfResult");
		if (null == crfFile || !crfFile.isFile()) {
			System.out.println("error in crf label! error message: ");
			BufferedReader brerror = new BufferedReader(isr);
			String errorLine = null;
			while ((errorLine = brerror.readLine()) != null) {
				System.out.println(errorLine);
			}
			return 10;
		}
		ArrayList<String> newAlign =
			general.orderAlign.labelToAlign.extractIndexLabel(this.featureFileName.split("\\.")[0] + ".crfResult");


		// to see if there are new seq that are no smaller than seqLengthLim found. 
		int newSeqNum = 0;
		// put the alignment found into the set of all the alignment found

		// found new alignments
		if (null != newAlign) {
			for (String align : newAlign) {
				allAlign.add(align);
			}
			// sort the new alignments into seqs 
			ArrayList<ArrayList<String>> newSeqs = general.orderAlign.longSeqSort.getExactGroups(newAlign);
			//if the seq length is larger than seqLengthLim, take it as a real alignment seq. 
			//and try to add it into currentAlign
			for (ArrayList<String> seq : newSeqs) {
				if (seq.size() >= longSeqLength) {
					newSeqNum += 1;
					for (String align : seq) {
						longSeqAlign.add(align);
					}
				}
			}
			// no new long seq found, but  many have been aligned, beam search end
			if (newSeqNum == 0 && this.alignedProcent(longSeqAlign) > this.longSeqAlignProcent) {
				return 10;
			}
		}

		//no new alignment is found,but 'enough ' alignments found, beam search stop.
		if ((null == newAlign || newAlign.size() == 0) && this.alignedProcent(longSeqAlign) > this.longSeqAlignProcent) {
			return 10;
		}

		//to note the largest distance in alignments whose source Id is larger than target Id 
		// if this value is much smaller than lastSEnd, no need to search in the area where source Id is much larger than target Id more
		//e.g.
		//lastSEnd=100, it means that the area where 0<sId-tId<100 has been searched
		//now we have maxSDiff=70, from alignment 500;430
		//it is not necessary to search the area with sId-tId>=100 any more.
		int maxSDiff = 0;
		//similar to maxSDiff
		int maxTDiff = 0;

		if (null != longSeqAlign) {
			//enough aligned, stop the beam search.

			if (this.alignedProcent(longSeqAlign) > alignedNumBorder) {
				return 3;
			}

			for (int i = 0; i < longSeqAlign.size(); i++) {
//			System.out.println("the size of exit alignment is "+currentAlign.size());
				String[] sourceStrings = longSeqAlign.get(i).split(";")[0].split(" ");
				String[] targetStrings = longSeqAlign.get(i).split(";")[1].split(" ");
				int sourceLastId = Integer.parseInt(sourceStrings[sourceStrings.length - 1]);
				int targetLastId = Integer.parseInt(targetStrings[targetStrings.length - 1]);

				int diff = sourceLastId - targetLastId;
				if (diff > 0 && maxSDiff < diff) {
					maxSDiff = diff;
				} else if (diff < 0 && -diff > maxTDiff) {
					maxTDiff = -diff;
				}
			}

		}

		// a large procent of the beam area from last time is not used, beam search will not be extended 
		// in middle of the diagonal
		if (maxSDiff * 1.0 / currentSEnd < beamUsedAreaProcent
			&& (currentTEnd != currentTStart && maxTDiff * 1.0 / currentTEnd < beamUsedAreaProcent)) {
			return 4;
		}
		// the area is below the diagonal
		if (maxSDiff * 1.0 / currentSEnd < beamUsedAreaProcent && (currentTEnd - currentTStart) <= 0) {
			return 4;
		}
		//the area is obove the diagonal
		if (maxTDiff * 1.0 / currentTEnd < beamUsedAreaProcent && (currentSEnd - currentSStart) <= 0) {
			return 4;
		}

		//get the new beam area
		// original no extension. 
		int newSStart = currentSEnd;
		int newSEnd = currentSEnd;
		int newTStart = currentTEnd;
		int newTEnd = currentTEnd;

		// the beam should extend from the source side.
		if (maxSDiff * 1.0 / currentSEnd >= beamUsedAreaProcent) {
			newSStart = currentSEnd;
			newSEnd = currentSEnd + (currentSEnd - currentSStart);
//    	   System.out.println("new source start and end is "+newSStart +" and "+newSEnd);

			this.extendBeam(
				newSStart, newSEnd, currentTEnd, currentTEnd, modelName);
		}

		// the beam extends from the target side.
		if (maxTDiff * 1.0 / currentTEnd >= beamUsedAreaProcent) {
			newTStart = currentTEnd;
			newTEnd = currentTEnd + (currentTEnd - currentTStart);
//    	   System.out.println("new target start and end is "+newTStart +" and "+newTEnd);
			this.extendBeam(
				currentSEnd, currentSEnd, newTStart, newTEnd, modelName);
		}
//       //alignment is in the middle of the area, but still not enough alignments are found, extend in both side
//       if(alignedProcent<0.5&&newSStart==newSEnd&&newTStart==newTEnd)
//       {
//    	   this.extendBeam( 
//					 currentSEnd,currentSEnd+Math.min(50,this.maxSId/20),currentTStart,currentTEnd,modelName);
//			this.extendBeam(
//					 currentSStart,currentSEnd,currentTEnd,currentTEnd+Math.min(50,this.maxTId/20),modelName);
//       }


		// the area where  the alignments are , is still not found.  
		//no alignment in the area that has been searched. extend the area in below and above the current . 
		if (null == longSeqAlign) {
			this.extendBeam(
				currentSEnd, currentSEnd + Math.min(50, this.maxSId / 20), currentTEnd, currentTEnd, modelName);
			this.extendBeam(
				currentSEnd, currentSEnd, currentTEnd, currentTEnd + Math.min(50, this.maxTId / 20), modelName);
		}
		return 9;
	}

	// get the dice for for each beam search iteration.
	public ArrayList<ArrayList<String>> getDice(ArrayList<String> existAlign, int sourceOldEnd, int sourceNewEnd, int targetOldEnd, int targetNewEnd) throws NumberFormatException, IOException {
		//the skip is out of range. 
		if (sourceNewEnd >= diceExtra.sourceAllSent.size()) {
			return null;
		}
		if (targetNewEnd >= diceExtra.targetAllSent.size()) {
			return null;
		}

//		System.out.println("sourceAllSent size is "+diceExtra.sourceAllSent.size());
//			System.out.println("targetAllSent size is "+diceExtra.targetAllSent.size());
		ArrayList<String> valueArray = new ArrayList<String>();
		ArrayList<String> indexArray = new ArrayList<String>();

		HashMap<Integer, Integer> alignedSource = this.alignedSource(existAlign);
		HashMap<Integer, Integer> alignedTarget = this.alignedTarget(existAlign);

		// the area below the diaginal, ie where source index  is larger than target index
		//if sourceOldEnd==sourceNewEnd, the search area is empty
		//this happens when the beam area moves to the top side .
		for (int belowDia = sourceOldEnd; belowDia < sourceNewEnd; belowDia++) {
			for (int seqNum = belowDia + 1; seqNum < diceExtra.sourceAllSent.size() + 1; seqNum++) {
				boolean aligned = this.hasAligned(alignedSource, alignedTarget, seqNum, seqNum - belowDia);
				if (aligned) {
					valueArray.add(null);
					indexArray.add(null);
					continue;
				}
				int[] sourceSent = getSent.getSingleSent(diceExtra.sourceAllSent, seqNum - 1);
				if (seqNum - belowDia >= diceExtra.targetAllSent.size()) {
					break;
				}
				int[] targetSent = getSent.getSingleSent(diceExtra.targetAllSent, seqNum - belowDia - 1);
				double diceValue = diceExtra.SentPairDice.get(sourceSent, targetSent);
				valueArray.add(doubleToString.toBinaryFeature(diceValue, 2) + "");
				indexArray.add(" " + (seqNum) + ";" + (seqNum - belowDia));
			}
			valueArray.add(null);
			indexArray.add(null);
		}

		// the area above  the diaginal, i.e. where source index  is smaller  than target index
		//if targetOldEnd==targetNewEnd, the search area is empty
		for (int aboveDia = targetOldEnd; aboveDia < targetNewEnd; aboveDia++) {
			for (int seqNum = aboveDia + 1; seqNum < diceExtra.targetAllSent.size() + 1; seqNum++) {
				boolean aligned = this.hasAligned(alignedSource, alignedTarget, seqNum - aboveDia, seqNum);
				if (aligned) {
					valueArray.add(null);
					indexArray.add(null);
					continue;
				}
				int[] targetSent = getSent.getSingleSent(diceExtra.targetAllSent, seqNum - 1);
				if (seqNum - aboveDia >= diceExtra.sourceAllSent.size()) {
					break;
				}
				int[] sourceSent = getSent.getSingleSent(diceExtra.sourceAllSent, seqNum - aboveDia - 1);

				double diceValue = diceExtra.SentPairDice.get(sourceSent, targetSent);
				valueArray.add(doubleToString.toBinaryFeature(diceValue, 2) + "");
				indexArray.add(" " + (seqNum - aboveDia) + ";" + seqNum);
			}
			valueArray.add(null);
			indexArray.add(null);
		}

		ArrayList<ArrayList<String>> returnArrays = new ArrayList<ArrayList<String>>();
		returnArrays.add(valueArray);
		returnArrays.add(indexArray);
		return returnArrays;
	}

	// get the dice for for each beam search iteration.
	public ArrayList<ArrayList<String>> getDiceTarget(ArrayList<String> existAlign, int targetOldEnd, int targetNewEnd) throws NumberFormatException, IOException {
		//the skip is out of range. 

		if (targetNewEnd >= diceExtra.targetAllSent.size()) {
			return null;
		}

//		System.out.println("sourceAllSent size is "+diceExtra.sourceAllSent.size());
//			System.out.println("targetAllSent size is "+diceExtra.targetAllSent.size());
		ArrayList<String> valueArray = new ArrayList<String>();
		ArrayList<String> indexArray = new ArrayList<String>();

		HashMap<Integer, Integer> alignedSource = this.alignedSource(existAlign);
		HashMap<Integer, Integer> alignedTarget = this.alignedTarget(existAlign);

		// the area above  the diaginal, i.e. where source index  is smaller  than target index
		//if targetOldEnd==targetNewEnd, the search area is empty
		for (int aboveDia = targetOldEnd; aboveDia < targetNewEnd; aboveDia++) {
			for (int seqNum = aboveDia + 1; seqNum < diceExtra.targetAllSent.size() + 1; seqNum++) {
				boolean aligned = this.hasAligned(alignedSource, alignedTarget, seqNum - aboveDia, seqNum);
				if (aligned) {
					valueArray.add(null);
					indexArray.add(null);
					continue;
				}
				int[] targetSent = getSent.getSingleSent(diceExtra.targetAllSent, seqNum - 1);
				if (seqNum - aboveDia >= diceExtra.sourceAllSent.size()) {
					break;
				}
				int[] sourceSent = getSent.getSingleSent(diceExtra.sourceAllSent, seqNum - aboveDia - 1);

				double diceValue = diceExtra.SentPairDice.get(sourceSent, targetSent);
				valueArray.add(doubleToString.toBinaryFeature(diceValue, 2) + "");
				indexArray.add(" " + (seqNum - aboveDia) + ";" + seqNum);
			}
			valueArray.add(null);
			indexArray.add(null);
		}

		ArrayList<ArrayList<String>> returnArrays = new ArrayList<ArrayList<String>>();
		returnArrays.add(valueArray);
		returnArrays.add(indexArray);
		return returnArrays;
	}
	// get the dice for for each beam search iteration.

	public ArrayList<ArrayList<String>> getDiceSource(ArrayList<String> existAlign, int sourceOldEnd, int sourceNewEnd) throws NumberFormatException, IOException {
		//the skip is out of range. 
		if (sourceNewEnd >= diceExtra.sourceAllSent.size()) {
			return null;
		}

//		System.out.println("sourceAllSent size is "+diceExtra.sourceAllSent.size());
//			System.out.println("targetAllSent size is "+diceExtra.targetAllSent.size());
		ArrayList<String> valueArray = new ArrayList<String>();
		ArrayList<String> indexArray = new ArrayList<String>();

		HashMap<Integer, Integer> alignedSource = this.alignedSource(existAlign);
		HashMap<Integer, Integer> alignedTarget = this.alignedTarget(existAlign);

		//beam area where sourceId>=target Id.
		//sourceNewEnd-sourceOldEnd decide the beam area, where the sourceNewEnd>=sourceId-targetId>sourceOld 
		for (int aroundDia = sourceOldEnd; aroundDia < sourceNewEnd; aroundDia++) {
			for (int seqNum = aroundDia + 1; seqNum < diceExtra.sourceAllSent.size() + 1; seqNum++) {
				boolean aligned = this.hasAligned(alignedSource, alignedTarget, seqNum, seqNum - aroundDia);
				if (aligned) {
					valueArray.add(null);
					indexArray.add(null);
					continue;
				}
				int[] sourceSent = getSent.getSingleSent(diceExtra.sourceAllSent, seqNum - 1);
				if (seqNum - aroundDia >= diceExtra.targetAllSent.size()) {
					break;
				}
				int[] targetSent = getSent.getSingleSent(diceExtra.targetAllSent, seqNum - aroundDia - 1);
				double diceValue = diceExtra.SentPairDice.get(sourceSent, targetSent);
				valueArray.add(doubleToString.toBinaryFeature(diceValue, 2) + "");
				indexArray.add(" " + (seqNum) + ";" + (seqNum - aroundDia));
			}
			valueArray.add(null);
			indexArray.add(null);
		}

		ArrayList<ArrayList<String>> returnArrays = new ArrayList<ArrayList<String>>();
		returnArrays.add(valueArray);
		returnArrays.add(indexArray);
		return returnArrays;
	}

	// to get all the source Ids that has already been aligned, so that in beam search, the pair contains this source sentence will
	//not be considered.
	public HashMap<Integer, Integer> alignedSource(ArrayList<String> existAlign) {
		if (null == existAlign) {
			return null;
		}
		HashMap<Integer, Integer> alignedSource = new HashMap<Integer, Integer>();
		for (String element : existAlign) {
			String[] sourceIds = element.split(";")[0].split("\\s+");
			for (String sourceId : sourceIds) {
				alignedSource.put(Integer.parseInt(sourceId), 1);
			}
		}

		return alignedSource;
	}

	// to get all the target Ids that has already been aligned, so that in beam search, the pair contains this target sentence will
	//not be considered.
	public HashMap<Integer, Integer> alignedTarget(ArrayList<String> existAlign) {
		if (null == existAlign) {
			return null;
		}
		HashMap<Integer, Integer> alignedTarget = new HashMap<Integer, Integer>();
		for (String element : existAlign) {
			String[] targetIds = element.split(";")[1].split("\\s+");
			for (String targetId : targetIds) {
				alignedTarget.put(Integer.parseInt(targetId), 1);
			}
		}

		return alignedTarget;
	}

	// to check if source or target has been aligned. if one of them has been aligned, 
	// return true, so that not necessary to get dice and other features for this sentence pair.
	public boolean hasAligned(HashMap<Integer, Integer> alignedSource, HashMap<Integer, Integer> alignedTarget, int source, int target) {
		if (null == alignedSource) {
			return false;
		}
		if (null != alignedSource.get(source)) {
			return true;
		}
		if (null != alignedTarget.get(target)) {
			return true;
		}

		return false;
	}

	/**
	 * get feature file for the current iteration of of extendBeam.
	 *
	 * @param diceArrays: this arrays save the dice feature of the current beam
	 * area. other features will be created according to the sentence index
	 * should in diceArrays. diceArrays.get(0) save the dice values ,
	 * diceArrays.get(1) save the corresponding indexes. there should be of the
	 * save size.
	 * @return
	 * @throws IOException
	 */
	public boolean getFeatureFile(ArrayList<ArrayList<String>> diceArrays) throws IOException {
		File featureFile = new File(this.featureFileName);
		BufferedWriter bufferedFW = new BufferedWriter(new FileWriter(featureFile));
		ArrayList<String> valueArray = diceArrays.get(0);
		ArrayList<String> indexArray = diceArrays.get(1);
		File diceFile = new File(this.featureFileName.split("\\.")[0] + ".dice");
		BufferedWriter br = new BufferedWriter(new FileWriter(diceFile));

		if (valueArray.size() == indexArray.size()) {

			for (int i = 0; i < valueArray.size(); i++) {
				if (null != valueArray.get(i)) {
					br.write(valueArray.get(i) + " ");
				}
				br.write(indexArray.get(i) + "\n");
			}
			br.close();
		} /*
		 * E
		 */ else {
			br.close();
		}

		try {
			for (int i = 0; i < valueArray.size(); i++) {
				if (null == valueArray.get(i) || null == indexArray.get(i)) {
					bufferedFW.write("\n");
				} else {
					int sId = Integer.parseInt(indexArray.get(i).split(";")[0].trim());
					int tId = Integer.parseInt(indexArray.get(i).split(";")[1].trim());

					String diceBinFeature = valueArray.get(i);

					//source sentence bean with neibours		
					sentenceBean sBean = sFileArray.getSentenceBean(sId);
					if (sBean == null) {
						for (int featureNum = 0; featureNum < 27; featureNum++) {
							bufferedFW.write("0");
							if (featureNum != 26) {
								bufferedFW.write(" ");
							}
						}
						bufferedFW.write("\t" + indexArray.get(i) + "\n");
						continue;

					}
					sentenceBean sBeanNeibour1;
					if (sId - 1 > 1) {
						sBeanNeibour1 = sFileArray.getSentenceBean(sId - 1);

					} else {
						sBeanNeibour1 = null;
					}

					sentenceBean sBeanNeibour2;
					if (sId - 2 > 1) {
						sBeanNeibour2 = sFileArray.getSentenceBean(sId - 2);
					} else {
						sBeanNeibour2 = null;
					}

					//target sentence bean with neibours
					sentenceBean tBean = tFileArray.getSentenceBean(tId);
					if (tBean == null) {
						for (int featureNum = 0; featureNum < 27; featureNum++) {
							bufferedFW.write("0");
							if (featureNum != 26) {
								bufferedFW.write(" ");
							}
						}
						bufferedFW.write("\t" + indexArray.get(i) + "\n");
						continue;

					}

					sentenceBean tBeanNeibour1;
					if (tId - 1 > 1) {
						tBeanNeibour1 = tFileArray.getSentenceBean(tId - 1);

					} else {
						tBeanNeibour1 = null;
					}

					sentenceBean tBeanNeibour2;
					if (tId - 2 > 1) {
						tBeanNeibour2 = tFileArray.getSentenceBean(tId - 2);
					} else {
						tBeanNeibour2 = null;
					}

					String[] feature1to1;
					String[] feature2to1;
					String[] feature3to1;
					String[] feature1to2;
					String[] feature1to3;

					sentenceBean sBean2 = sBean.addBean(sBeanNeibour1);
					sentenceBean sBean3 = sBean2.addBean(sBeanNeibour2);

					sentenceBean tBean2 = tBean.addBean(tBeanNeibour1);
					sentenceBean tBean3 = tBean2.addBean(tBeanNeibour2);

					beanPair BeanPair = new beanPair();

					feature1to1 = BeanPair.pairFeature(sBean, tBean, posTable);
					feature1to2 = BeanPair.pairFeature(sBean2, tBean, posTable);
					feature1to3 = BeanPair.pairFeature(sBean3, tBean, posTable);
					feature2to1 = BeanPair.pairFeature(sBean, tBean2, posTable);
					feature3to1 = BeanPair.pairFeature(sBean, tBean3, posTable);
					String positionFeature = this.getPositionRatio(sId, tId);

					bufferedFW.write(feature1to1[0] + " ");
					bufferedFW.write(feature1to1[1] + " ");
					bufferedFW.write(feature1to1[2] + " ");
					bufferedFW.write(feature1to1[3] + " ");
					bufferedFW.write(feature1to1[4] + " ");
					bufferedFW.write(positionFeature + " ");
					bufferedFW.write(diceBinFeature + " ");

					for (int j = 0; j < 5; j++) {
						bufferedFW.write(feature2to1[j] + " ");
					}

					for (int j = 0; j < 5; j++) {
						bufferedFW.write(feature3to1[j] + " ");
					}

					for (int j = 0; j < 5; j++) {
						bufferedFW.write(feature1to2[j] + " ");
					}

					for (int j = 0; j < 5; j++) {
						if (j != 4) {
							bufferedFW.write(feature1to3[j] + " ");
						} else {
							bufferedFW.write(feature1to3[4]);
						}
					}

					bufferedFW.write("\t" + indexArray.get(i));
					bufferedFW.write("\n");
				}
			}
			bufferedFW.flush();
			bufferedFW.close();
			return true;
		} catch (Exception o) {
			o.printStackTrace();
		}
		return false;
	}

	public String getPositionRatio(int sId, int tId) {
		double positionRatio = Math.abs(sId * 1.0 / this.maxSId - tId * 1.0 / this.maxTId);
//		 System.out.println("double :"+positionRatio);
		String binaryPosition = String.format("%1.1e", positionRatio);
//		 System.out.println("string:"+binaryPosition);
		return binaryPosition;
	}

// get the procent of sentence pairs that have been aligned.	 
	public double alignedProcent(ArrayList<String> currentAlign) {
		int sourceAlignedNum = this.alignedSource(currentAlign).size();
		int targetAlignedNum = this.alignedTarget(currentAlign).size();
		return Math.max(sourceAlignedNum * 1.0 / this.maxSId, targetAlignedNum * 1.0 / this.maxTId);
	}

	// remove the temporary file.
	public void removeFile(String fileName) throws IOException, InterruptedException {
		Process remover = Runtime.getRuntime().exec("rm " + fileName);
		remover.waitFor();
		return;
	}

	public int isArrayEle(ArrayList<String> list, String element) {
		if (list.size() == 0) {
			return 0;
		} else {
			for (String listEle : list) {
				if (listEle.trim().equals(element)) {
					return 1;
				}
			}
			return 0;
		}
	}

	public void writeAlignments(
		String extension //E
		) throws IOException, InterruptedException {
		String[] pathString = this.sourceName.split("/");

		String alignPath = "/";
		for (int i = 0; i < pathString.length - 2; i++) {
			alignPath += pathString[i] + "/";
		}
		alignPath += "align/";
//		 System.out.println("align path is "+alignPath);
		File alignDir = new File(alignPath);
		if (!alignDir.isDirectory()) {
			boolean success = alignDir.mkdirs();
			if (!success) {
				System.out.println("unsuccessuful in creatting the alignment directory!");
				return;
			}
		}
		if (allAlign.size() == 0 || null == allAlign) {
			return;
		}

		String fileName = pathString[pathString.length - 1].split("\\.")[0];

//		 ArrayList<ArrayList<String>> allSeqs=longSeqSort.getExactGroups(allAlign);
		ArrayList<String> lastAlign = longSeqSort.arrayToAligns(allAlign);
//ORIG		 writeArray.writeOne(alignPath+fileName+".align", lastAlign);
		writeArray.writeOne(alignPath + fileName + extension, lastAlign); //E

		this.removeFile(this.featureFileName.split("\\.")[0] + ".crfResult");
//		 this.removeFile(this.featureFileName);
		System.out.println("the file pair (" + this.sourceName + "/" + this.targetName + ") are aligned. "
			+ "and the result is saved under " + alignPath);
	}
}
