/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package generalTrain;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import general.com.flexEdit.ExtractFlexibleEditDistance;

/** Sentence pairs with their features.
 *
 * @author Huiqin Körkel-Qu
 */
public class beanPair {

	public double lengthRatio = 0;
	public double positionRatio = 0;
	public int punctEqual = 0;
	public double posAgreenment = 0;
	public int orthAgreenment = 0;
	public double flexEditDistance = 0;
//    ArrayList<Integer> sIdList;
//    ArrayList<Integer> tIdList;
	int sId;
	int tId;
	int s1Length;
	int s2Length;
	sentenceBean sBean, tBean;
	Hashtable<String, ArrayList<String>> posMap;

	public String[] pairFeature(sentenceBean sBean1, sentenceBean sBean2, Hashtable<String, ArrayList<String>> posMap) {

//		this.sIdList=sBean1.sentenceIds;
//	    this.tIdList=sBean2.sentenceIds;

		if (null == sBean1 || null == sBean2) {
			return null;
		} else {

			this.sBean = sBean1;
			this.tBean = sBean2;
			this.posMap = posMap;
			this.sId = sBean1.sentenceId;
			this.tId = sBean2.sentenceId;

			int s1Length = sBean1.lemmaWordSeq.length;
			int s2Length = sBean2.lemmaWordSeq.length;
			this.s1Length = s1Length;
			this.s2Length = s2Length;

			posAgreenment = this.setPosAgreenment(sBean1.posSeq, sBean2.posSeq, posMap);
			//System.out.println("the pos Agreenment is "+posAgreenment);
			orthAgreenment = this.setOrthAgreenment(sBean1.lemmaWordSeq, sBean2.lemmaWordSeq);
			//System.out.println("the orthAgreenment is "+orthAgreenment);
			punctEqual = this.setPunctEqual(sBean1.lemmaWordSeq[s1Length - 1], sBean2.lemmaWordSeq[s2Length - 1]);
			//System.out.println("the punctEqual is "+punctEqual);
			lengthRatio = this.setLengthRatio(s1Length, s2Length);

			flexEditDistance = this.setFlexEditDistance(sBean1.lemmaWordSeq, sBean2.lemmaWordSeq);
		}
		String[] featureValue = new String[5];
		featureValue[0] = this.toBinaryFeature(lengthRatio, 2, "normal");
		featureValue[1] = this.toBinaryFeature(posAgreenment, 2, "normal");
		featureValue[2] = orthAgreenment + "";
		featureValue[3] = punctEqual + "";
		featureValue[4] = this.toBinaryFeature(flexEditDistance, 2, "normal");

		return featureValue;

	}//end of constructor method pairFeature

	public String toBinaryFeature(double inValue, int bitNum, String type) {
		//inValue is the value to be round, bitNum is how many bit will be keep after .

		if (type.equals("normal")) // for a value larger than 0.09999999999, use  intpart+ "-" + decimalpart
		{
			int expand = 1;
			for (int i = 0; i < bitNum; i++) {
				expand *= 10;
			}
			double expandValue = inValue * expand;
			//from http://topic.csdn.net/t/20041201/09/3604201.html
			java.text.NumberFormat formate = java.text.NumberFormat.getNumberInstance();
			formate.setMaximumFractionDigits(0);
			String output = formate.format(expandValue);

			return output.split("\\.")[0];
		} // for the value which is too large or too small, use scientific value. 
		else if (type.equals("scienceNum")) {

			String binaryFeature = String.format("%1.1e", inValue);
			return binaryFeature;
		} //this should not happen!
		else {
			return "0";
		}
	}

	public double setLengthRatio(int s1Length, int s2Length) {
		double lengthRatio = 0.0;
		try {
			lengthRatio = Math.min(s1Length * 1.0 / s2Length, s2Length * 1.0 / s1Length);
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("error in setLengthRatio");
		}
		return lengthRatio;
	}

	//punctuation equal decision
	public int setPunctEqual(String sBean1Punct, String sBean2Punct) {
		int punctEqual = 0;
		try {
			if (sBean1Punct.equals(sBean2Punct)) //level of punctequal, ". ,  ? !  other"
			{
				if (sBean1Punct.equals(".")) {
					punctEqual = 1;
				} else if (sBean1Punct.equals(",")) {
					punctEqual = 2;
				} else if (sBean1Punct.equals("?")) {
					punctEqual = 3;
				} else if (sBean1Punct.equals("!")) {
					punctEqual = 3;
				} else {
					punctEqual = 4;
				}
			} else {
				punctEqual = 0;
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("error in punctEqualdecsion");
		}
		return punctEqual;
	}

	public double setPosAgreenment(String[] sBean1Pos, String[] sBean2Pos, Hashtable<String, ArrayList<String>> posTable) {
		double posAgreenment = 0.0;
		int[] tIsUsed = new int[sBean2Pos.length];
		//set all the soure tags and target tags as not used for mapping
		// 0 for not used 
		// 1 for  used

		for (int i = 0; i < sBean2Pos.length; i++) {
			tIsUsed[i] = 0;
		}

		for (int i = 0; i < sBean1Pos.length; i++) {
			boolean iPosIsFound = false;
			ArrayList<String> TPosInHash;

			TPosInHash = (ArrayList<String>) posTable.get(sBean1Pos[i]);

			// if there is not corresponding tag in the maptable, give this tag a PosAgreenment 0.3
			// PTKA,PAV,PRF,APZR,(also TRUNG)
			if ((null == TPosInHash || TPosInHash.get(0).equals(""))) {
				posAgreenment += 0.3;
				continue;
			} else {
				//below is the case that target tag(s) exist(s).

				for (int j = 0; j < sBean2Pos.length; j++) {
					if (tIsUsed[j] == 1) {
						continue;
					}

					for (int k = 0; k < TPosInHash.size(); k++) {
						if (TPosInHash.get(k).equals(sBean1Pos[i])) {//sIsUsed[i]=1;
							tIsUsed[j] = 1;
							posAgreenment += 1;
							iPosIsFound = true;
							break;
						}
					}

					if (iPosIsFound) {
						break;
					}
				}
			}
		}

		//return posAgreenment/sBean1Pos.length;
		if (posAgreenment * 2 / (sBean1Pos.length + sBean2Pos.length) > 1) {
			System.out.println("posAgreement larger than 1");
			System.out.println(posAgreenment + ";" + sBean1Pos.length + ";" + sBean2Pos.length);
			for (int i = 0; i < sBean1Pos.length; i++) {
				System.out.print(sBean1Pos[i]);
			}

			for (int i = 0; i < sBean2Pos.length; i++) {
				System.out.print(sBean2Pos[i]);
				if (i == sBean2Pos.length - 1) {
					System.out.println();
				}
			}
		}

		return posAgreenment * 2 / (sBean1Pos.length + sBean2Pos.length);
	}

	//end of setPosAgreenment
	//to see how many special words are the same in the source and target sentences.
	public int setOrthAgreenment(String[] sBean1Lemma, String[] sBean2Lemma) {
		int orthAgreenment = 0;
		Pattern pattern = Pattern.compile("^[a-z]"); //E bug? "[^a-z]" should be ???
		Matcher matcher;
		boolean isSpecWord = false;
		boolean[] isUsed = new boolean[this.s2Length];
		for (int i = 0; i < isUsed.length; i++) {
			isUsed[i] = false;
		}

		//to decide every word in source to see if it is a special word
		for (int i = 0; i < s1Length; i++) {
			isSpecWord = false;
			for (int j = 0; j < sBean1Lemma[i].length(); j++) {
				matcher = pattern.matcher(String.valueOf(sBean1Lemma[i].charAt(j)));
				if (matcher.matches() == true) {
					isSpecWord = true;
					break;
				}
			}
			//if the source word is special word, search for the mapping in target sentenceBean
			//if there is the map, increase the orthAgreenment. 

			if (isSpecWord == true) {
				for (int j = 0; j < this.s2Length; j++) {
					if (sBean1Lemma[i].toLowerCase().equals(sBean2Lemma[j].toLowerCase()) && !isUsed[j]) {
						isUsed[j] = true;
						if (sBean2Lemma[j].equals(",")) {
							orthAgreenment += 1;
						} else if (sBean2Lemma[j].equals(".")) {
							orthAgreenment += 1;
						} else {
							orthAgreenment += 2;
						}
						break;
					}
				}
			}
		}

		return orthAgreenment;
	}

	public double setFlexEditDistance(String[] sLemma, String[] tLemma) {
		//List<String> list=Arrays.asList(sLemma);
		ArrayList<String> sList = new ArrayList<String>();
		for (int i = 0; i < sLemma.length; i++) {
			sList.add(sLemma[i]);
		}

		ArrayList<String> tList = new ArrayList<String>();
		for (int i = 0; i < tLemma.length; i++) {
			tList.add(tLemma[i]);
		}
		double flexDistance = ExtractFlexibleEditDistance.computeFlexibleEditDistance(sList, tList, 0.8);
		return flexDistance;
	}
}
