/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package generalTrain;

import java.io.File;
import java.util.List;

import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;

/**
 * change the xml file into array of sentence Beans.
 *
 * @author Huiqin Körkel-Qu
 */
public class fileToArray {

	String fileTitle = new String();
	File f;
	SAXReader reader;
	List<Node> nodes;
	public sentenceBean[] allBeans;
	int maxId;
	//according to the fileTitle, get the document of file

	@SuppressWarnings("unchecked")
	public fileToArray(String title) {
//    	System.out.println(title);
		try { //this two lines add the sentence id in the xml file.
			//addSentenceId addIdCurrentFile=new addSentenceId(title);
			//addIdCurrentFile.addId();

			fileTitle = title;
//		f=new File(fileTitle);
			reader = new SAXReader();
			Document doc = reader.read(fileTitle);
			doc.setXMLEncoding("ISO-8859-1");
			try {
				List<Node> allNodes = doc.selectNodes("//sentences/s");
				nodes = allNodes;
			} catch (Exception e) {
				e.printStackTrace();
			}

		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("xmlreadererror");
		}
		this.getAllSentenceBeans();
	}

	// get the maxId of sentences in this file, for feature positionRatio.
	public int getMaxId() {

		for (Node node : nodes) {
			Attribute idAttr = ((Element) node).attribute("id");
			int currentId = Integer.parseInt(idAttr.getValue());
			if (currentId > maxId) {
				maxId = currentId;
			}
		}

		return maxId;
	}

//    change all the sentence into sentence Beans and get 
//    the array of sentence Beans. 
//    each bean represents a sentence, which contains the original word sequences, the word sequence after lemmata,
//    and the POS sequence,(these three sequences are saved as array of strings.), sentence Id(s), the number of words,
//    the last punctuation of the sentences.
	public void getAllSentenceBeans() {
		this.getMaxId();
		allBeans = new sentenceBean[maxId];

		for (Node node : nodes) {
			Attribute idAttr = ((Element) node).attribute("id");
			int sentenceId = Integer.parseInt(idAttr.getValue());
//  	      System.out.println("currentId is "+sentenceId);
			int wordNum = 0;
			String[] wordSeq = null;
			String[] posSeq = null;
			String[] lemmaWordSeq = null;
			sentenceBean sentenceInfo = new sentenceBean();

			if (((Element) node).getName() != ("s")) {
				System.out.println("there is other attribute except \"s\" in the xml file!");
				continue;
			}
			// sentenceWPL is the text between <s> and </s> which shows each word and its pos and lemma in the sentences
			String sentenceWPL = ((Element) node).getText().toString();

			wordNum = sentenceWPL.split("\n").length - 1;
			if (wordNum <= 0) {
				allBeans[sentenceId - 1] = null;
				continue;
			}

			String[] wordWPL = new String[wordNum];
			wordSeq = new String[wordNum];
			posSeq = new String[wordNum];
			lemmaWordSeq = new String[wordNum];

			for (int i = 1; i < wordNum + 1; i++) {
				wordWPL[i - 1] = sentenceWPL.split("\n")[i];

				wordSeq[i - 1] = wordWPL[i - 1].split("\t")[0].trim();
				posSeq[i - 1] = wordWPL[i - 1].split("\t")[1].trim();
				lemmaWordSeq[i - 1] = wordWPL[i - 1].split("\t")[2].trim();
			}

			sentenceInfo.setSentenceLength(wordNum);
			sentenceInfo.setSentenceId(sentenceId);
			sentenceInfo.setSentencePos(posSeq);
			sentenceInfo.setSentenceLemmaWord(lemmaWordSeq);
			if (posSeq[wordNum - 1].substring(0, 1).equals("$")) {	 //to decide if the last "word" is a word or a punctuation.
				sentenceInfo.setSentencePunct(wordSeq[wordSeq.length - 1]);
				// System.out.println("the last word is a punct");
			} else {//System.out.println("the last word is not a punct");
			}
			allBeans[sentenceId - 1] = sentenceInfo;

		}
	}

//    get a single sentence bean out according to the sentence Id
	public sentenceBean getSentenceBean(int sentenceId) {

		try {
			return allBeans[sentenceId - 1];
		} catch (Exception o) {
			o.printStackTrace();
			System.out.println("the file with error is:" + fileTitle);
			System.out.println("the " + sentenceId + " sentence  has no beans!");
			return null;
		}
	}
}
